# -*- encoding: utf-8 -*-
##############################################################################
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
##############################################################################


from openerp.osv import fields, osv
from openerp.tools.translate import _

class faktur_pajak(osv.osv):
    _name = 'pajak.faktur_pajak'
    _inherit = 'pajak.faktur_pajak'

    
    _columns =  {
                'journal_id' : fields.many2one(string='Journal',
                    obj='account.journal',
                    states={'draft':[('readonly',False)]},
                    readonly=True,
                    ),
                'account_id' : fields.many2one(string='Account',
                    obj='account.account',
                    states={'draft':[('readonly',False)]},
                    readonly=True,
                    ),
                'advance_payment_account_id' : fields.many2one(string='Account',
                    obj='account.account',
                    states={'draft':[('readonly',False)]},
                    readonly=True,
                    ),
                'move_id' : fields.many2one(string='# Move',
                    obj='account.move',
                    readonly=True,
                    ),
                'move_line_id' : fields.many2one(string='Move Line',
                    obj='account.move.line',
                    readonly=True,
                    ),
                }   

    def workflow_action_post(self, cr, uid, ids, context={}):
        super(faktur_pajak, self).workflow_action_post(cr, uid, ids, context)

        for id in ids:
            if not self._create_accounting_entry(cr, uid, id):
                return False

        return False

    def workflow_action_cancel(self, cr, uid, ids, context={}):
        super(faktur_pajak, self).workflow_action_cancel(cr, uid, ids, context)

        for id in ids:
            if not self._cancel_account_move(cr, uid, id):
                return False

        return False

    def _will_create_acc_entry(self, cr, uid, id):
        fp = self.browse(cr, uid, [id])[0]

        if fp.journal_id:
            return True
        else:
            return False

    def _create_accounting_entry(self, cr, uid, id):
        obj_line = self.pool.get('pajak.faktur_pajak_line')
        obj_ppnbm = self.pool.get('pajak.faktur_pajak_ppnbm_line')
        obj_move = self.pool.get('account.move')

        if not self._will_create_acc_entry(cr, uid, id):
            return True

        fp = self.browse(cr, uid, [id])[0]

        move_id = self._create_acc_move(cr, uid, id)

        if not move_id:
            return False

        if not self._create_move_line(cr, uid, id, move_id):
            return False

        if fp.advance_payment > 0.0:
            if not self._create_advance_payment_move_line(cr, uid, id, move_id):
                return False

        for line in fp.faktur_pajak_line_ids:
            if not obj_line._create_move_line(cr, uid, line.id, move_id):
                return False

        for line_ppnbm in fp.faktur_pajak_line_ppnbm_ids:
            if not obj_ppnbm._create_move_line(cr, uid, line_ppnbm.id, move_id):
                return False

        obj_move.post(cr, uid, [move_id])

        self.write(cr, uid, [id], {'move_id' : move_id})

        return True

    def _create_acc_move(self, cr, uid, id):
        obj_account_move = self.pool.get('account.move')
        obj_period = self.pool.get('account.period')

        account_move_id = False

        fp = self.browse(cr, uid, [id])[0]

        period_ids = obj_period.find(cr, uid, fp.faktur_pajak_date) #TODO

        if period_ids:
            period_id = period_ids[0]


        res =   {
                'name' : fp.name,
                'date' : fp.faktur_pajak_date,
                'journal_id' : fp.journal_id.id,
                'period_id' : period_id,
                'origin_type' : 'oe_doc',
                'origin_oe_doc_id' : 'pajak.faktur_pajak,%s' % (fp.id),
                }

        account_move_id = obj_account_move.create(cr, uid, res)

        return account_move_id
    

    def _cancel_account_move(self, cr, uid, id):
        obj_move = self.pool.get('account.move')

        fp = self.browse(cr, uid, [id])[0]

        if fp.move_id:
            obj_move.button_cancel(cr, uid, [fp.move_id.id])
            obj_move.unlink(cr, uid, [fp.move_id.id])

        return True

    def _create_move_line(self, cr, uid, id, account_move_id):
        obj_account_move_line = self.pool.get('account.move.line')

        fp = self.browse(cr, uid, [id])[0]

        debit = 0.0
        credit = 0.0
        partner_id = False


        if fp.jenis_fp == 'keluaran':
            partner_id = fp.buyer_partner_id.id
            debit = fp.amount_tax
        else:
            credit = fp.amount_tax
            partner_id = fp.seller_partner_id.id

        res =   {
                'name' : fp.name,
                'debit' : debit,
                'credit' : credit,
                'account_id' : fp.account_id.id,
                'move_id' : account_move_id,
                'partner_id' : partner_id,
                'analytics_id' : False, #TODO
                'origin_type' : 'oe_doc',
                'origin_oe_doc_id' : 'pajak.faktur_pajak,%s' % (fp.id),
                'direct_analytic_ids' : [],
                'indirect_analytic_ids' : []
                }

        move_id = obj_account_move_line.create(cr, uid, res)

        self.write(cr, uid, [fp.id], {'move_line_id' : move_id})
        return True
 

    def _create_advance_payment_move_line(self, cr, uid, id, account_move_id):
        obj_account_move_line = self.pool.get('account.move.line')

        fp = self.browse(cr, uid, [id])[0]

        debit = 0.0
        credit = 0.0
        partner_id = False


        if fp.jenis_fp == 'keluaran':
            partner_id = fp.buyer_partner_id.id
            debit = 0.1 * fp.advance_payment
        else:
            credit = 0.1 * fp.advance_payment
            partner_id = fp.seller_partner_id.id

        res =   {
                'name' : _('Advance payment contra-entry for %s') % fp.name,
                'debit' : debit,
                'credit' : credit,
                'account_id' : fp.advance_payment_account_id.id,
                'move_id' : account_move_id,
                'partner_id' : partner_id,
                'analytics_id' : False, #TODO
                'origin_type' : 'oe_doc',
                'origin_oe_doc_id' : 'pajak.faktur_pajak,%s' % (fp.id),
                'direct_analytic_ids' : [],
                'indirect_analytic_ids' : []
                }

        move_id = obj_account_move_line.create(cr, uid, res)

        return True

faktur_pajak()

class faktur_pajak_line(osv.osv):
    _name = 'pajak.faktur_pajak_line'
    _inherit = 'pajak.faktur_pajak_line'
    _description = 'Faktur Pajak Line'
    
    _columns =  {
                'account_id' : fields.many2one(string='Account',
                    obj='account.account',
                    ),
                'move_id' : fields.many2one(string='Move Line',
                    obj='account.move.line',
                    ),
                }   


    def _create_move_line(self, cr, uid, id, account_move_id):
        obj_account_move_line = self.pool.get('account.move.line')

        line = self.browse(cr, uid, [id])[0]
        fp = line.faktur_pajak_id

        debit = 0.0
        credit = 0.0
        partner_id = False


        if fp.jenis_fp == 'masukan':
            partner_id = fp.seller_partner_id.id
            debit = line.subtotal * 0.1
        else:
            credit = line.subtotal * 0.1
            partner_id = fp.buyer_partner_id.id

        res =   {
                'name' : line.name,
                'debit' : debit,
                'credit' : credit,
                'account_id' : line.account_id.id,
                'move_id' : account_move_id,
                'partner_id' : partner_id,
                'analytics_id' : False, #TODO
                'origin_type' : 'oe_doc',
                'origin_oe_doc_id' : 'pajak.faktur_pajak_line,%s' % (line.id),
                'direct_analytic_ids' : [],
                'indirect_analytic_ids' : []
                }

        move_id = obj_account_move_line.create(cr, uid, res)

        self.write(cr, uid, [id], {'move_id' : move_id})

        return True

faktur_pajak_line()

class faktur_pajak_ppnbm_line(osv.osv):
    _name = 'pajak.faktur_pajak_ppnbm_line'
    _inherit = 'pajak.faktur_pajak_ppnbm_line'
    _description = 'Faktur Pajak PPNBm Line'

    
    _columns =  {
                'account_id' : fields.many2one(string='Account',
                    obj='account.account',
                    ),
                'move_id' : fields.many2one(string='Move Line',
                    obj='account.move.line',
                    ),
                }   

    def _create_move_line(self, cr, uid, id, account_move_id):
        obj_account_move_line = self.pool.get('account.move.line')

        line = self.browse(cr, uid, [id])[0]
        fp = line.faktur_pajak_id

        debit = 0.0
        credit = 0.0
        partner_id = False


        if fp.jenis_fp == 'masukan':
            partner_id = fp.seller_partner_id.id
            debit = fp.amount_tax
        else:
            credit = fp.amount_tax
            partner_id = fp.buyer_partner_id.id

        res =   {
                'name' : 'PPNBM %s ' % str(line.rate),
                'debit' : debit,
                'credit' : credit,
                'account_id' : line.account_id.id,
                'move_id' : account_move_id,
                'partner_id' : partner_id,
                'analytics_id' : False, #TODO
                'origin_type' : 'oe_doc',
                'origin_oe_doc_id' : 'pajak.faktur_pajak_ppnbm_line,%s' % (line.id),
                'direct_analytic_ids' : [],
                'indirect_analytic_ids' : []
                }

        move_id = obj_account_move_line.create(cr, uid, res)

        self.write(cr, uid, [id], {'move_id' : move_id})

        return True
faktur_pajak_ppnbm_line()

class faktur_pajak_keluaran(osv.osv):
    _name = 'pajak.faktur_pajak_keluaran'
    _inherit = 'pajak.faktur_pajak'
    _table = 'pajak_faktur_pajak'
    _description = 'Faktur Pajak Keluaran'


    def _workflow_trigger(self, cr, uid, ids, trigger, context=None):
        #override in order to trigger the workflow of account.voucher at the end of create, write and unlink operation
        #instead of it's own workflow (which is not existing)
        return self.pool.get('pajak.faktur_pajak')._workflow_trigger(cr, uid, ids, trigger, context=context)

    def _workflow_signal(self, cr, uid, ids, signal, context=None):
        #override in order to fire the workflow signal on given account.voucher workflow instance
        #instead of it's own workflow (which is not existing)
        return self.pool.get('pajak.faktur_pajak')._workflow_signal(cr, uid, ids, signal, context=context)
        


faktur_pajak_keluaran()

class faktur_pajak_masukan(osv.osv):
    _name = 'pajak.faktur_pajak_masukan'
    _inherit = 'pajak.faktur_pajak'
    _table = 'pajak_faktur_pajak'
    _description = 'Faktur Pajak Masukan'


    def _workflow_trigger(self, cr, uid, ids, trigger, context=None):
        #override in order to trigger the workflow of account.voucher at the end of create, write and unlink operation
        #instead of it's own workflow (which is not existing)
        return self.pool.get('pajak.faktur_pajak')._workflow_trigger(cr, uid, ids, trigger, context=context)

    def _workflow_signal(self, cr, uid, ids, signal, context=None):
        #override in order to fire the workflow signal on given account.voucher workflow instance
        #instead of it's own workflow (which is not existing)
        return self.pool.get('pajak.faktur_pajak')._workflow_signal(cr, uid, ids, signal, context=context)
        


faktur_pajak_masukan()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
